@@ -20,6 +20,7 @@ import (
 	"context"
 	"fmt"
 	"io"
+	"os"
 
 	"github.com/spf13/cobra"
 
@@ -35,7 +36,8 @@ import (
 )
 
 var (
-	yamlOnly bool
+	yamlOnly   bool
+	outputFile string
 	// for testing
 	getRunContext = runcontext.GetRunContext
 	getCfgs       = parser.GetAllConfigs
@@ -49,7 +51,9 @@ func NewCmdDiagnose() *cobra.Command {
 		WithExample("Print the effective skaffold.yaml configuration for given profile", "diagnose --yaml-only --profile PROFILE").
 		WithCommonFlags().
 		WithFlags([]*Flag{
-			{Value: &yamlOnly, Name: "yaml-only", DefValue: false, Usage: "Only prints the effective skaffold.yaml configuration"}}).
+			{Value: &yamlOnly, Name: "yaml-only", DefValue: false, Usage: "Only prints the effective skaffold.yaml configuration"},
+			{Value: &outputFile, Name: "output", Shorthand: "o", DefValue: "", Usage: "File to write diagnose result"},
+		}).
 		NoArgs(doDiagnose)
 }
 
@@ -60,6 +64,15 @@ func doDiagnose(ctx context.Context, out io.Writer) error {
 	if err != nil {
 		return err
 	}
+	if outputFile != "" {
+		f, err := os.Create(outputFile)
+		if err != nil {
+			return err
+		}
+		defer f.Close()
+		out = f
+	}
+
 	if !yamlOnly {
 		if err := printArtifactDiagnostics(ctx, out, configs); err != nil {
 			return err
