@@ -118,11 +118,14 @@ func (v *Verifier) TrackContainerFromBuild(artifact graph.Artifact, container tr
 // from each specified image, executing them, and waiting for execution to complete.
 func (v *Verifier) Verify(ctx context.Context, out io.Writer, allbuilds []graph.Artifact) error {
 	var err error
-	v.once.Do(func() {
-		err = v.client.NetworkCreate(ctx, v.network)
-	})
-	if err != nil {
-		return fmt.Errorf("creating skaffold network %s: %w", v.network, err)
+
+	if !v.networkFlagPassed {
+		v.once.Do(func() {
+			err = v.client.NetworkCreate(ctx, v.network)
+		})
+		if err != nil {
+			return fmt.Errorf("creating skaffold network %s: %w", v.network, err)
+		}
 	}
 
 	builds := []graph.Artifact{}
@@ -184,7 +187,8 @@ func (v *Verifier) createAndRunContainer(ctx context.Context, out io.Writer, art
 
 	if container, found := v.tracker.ContainerForImage(artifact.ImageName); found {
 		olog.Entry(ctx).Debugf("removing old container %s for image %s", container.ID, artifact.ImageName)
-		if err := v.client.Delete(ctx, out, container.ID); err != nil {
+		v.client.Stop(ctx, container.ID, nil)
+		if err := v.client.Remove(ctx, container.ID); err != nil {
 			return fmt.Errorf("failed to remove old container %s for image %s: %w", container.ID, artifact.ImageName, err)
 		}
 		v.portManager.RelinquishPorts(container.Name)
@@ -253,7 +257,8 @@ func (v *Verifier) createAndRunContainer(ctx context.Context, out io.Writer, art
 	case <-time.After(v.testTimeout):
 		// verify test timed out
 		containerErr = errors.New(fmt.Sprintf("%q running container image %q timed out after : %s", opts.VerifyTestName, opts.ContainerConfig.Image, v.testTimeout))
-		err := v.client.Delete(ctx, out, id)
+		v.client.Stop(ctx, id, nil)
+		err := v.client.Remove(ctx, id)
 		if err != nil {
 			return errors.Wrap(containerErr, err.Error())
 		}
@@ -308,8 +313,8 @@ func (v *Verifier) Cleanup(ctx context.Context, out io.Writer, dryRun bool) erro
 		return nil
 	}
 	for _, container := range v.tracker.DeployedContainers() {
-		if err := v.client.Delete(ctx, out, container.ID); err != nil {
-			// TODO(nkubala): replace with actionable error
+		v.client.Stop(ctx, container.ID, nil)
+		if err := v.client.Remove(ctx, container.ID); err != nil {
 			olog.Entry(ctx).Debugf("cleaning up deployed container: %s", err.Error())
 		}
 		v.portManager.RelinquishPorts(container.ID)
